@@ -22,6 +22,7 @@ from six import iteritems
 from six.moves.urllib import parse as urllib_parse
 from webob import exc
 
+from tacker._i18n import _
 from tacker.common import constants
 from tacker.common import exceptions
 from tacker import wsgi
@@ -84,7 +85,7 @@ def get_limit_and_marker(request):
                     pagination, then return None.
     """
     max_limit = _get_pagination_max_limit()
-    limit = _get_limit_param(request, max_limit)
+    limit = _get_limit_param(request)
     if max_limit > 0:
         limit = min(max_limit, limit) or max_limit
     if not limit:
@@ -108,15 +109,16 @@ def _get_pagination_max_limit():
     return max_limit
 
 
-def _get_limit_param(request, max_limit):
+def _get_limit_param(request):
     """Extract integer limit from request or fail."""
+    limit = -1
     try:
         limit = int(request.GET.get('limit', 0))
         if limit >= 0:
             return limit
     except ValueError:
         pass
-    msg = _("Limit must be an integer 0 or greater and not '%d'")
+    msg = _("Limit must be an integer 0 or greater and not '%d'") % limit
     raise exceptions.BadRequest(resource='limit', msg=msg)
 
 
@@ -274,7 +276,17 @@ class SortingEmulatedHelper(SortingHelper):
     def sort(self, items):
         def cmp_func(obj1, obj2):
             for key, direction in self.sort_dict:
-                ret = cmp(obj1[key], obj2[key])
+                o1 = obj1[key]
+                o2 = obj2[key]
+
+                if o1 is None and o2 is None:
+                    ret = 0
+                elif o1 is None and o2 is not None:
+                    ret = -1
+                elif o1 is not None and o2 is None:
+                    ret = 1
+                else:
+                    ret = (o1 > o2) - (o1 < o2)
                 if ret:
                     return ret * (1 if direction else -1)
             return 0
