@@ -15,16 +15,17 @@
 #    under the License.
 
 import abc
-import imp
 import os
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import importutils
 import routes
 import six
 import webob.dec
 import webob.exc
 
+from tacker._i18n import _
 from tacker.common import exceptions
 import tacker.extensions
 from tacker import policy
@@ -166,7 +167,7 @@ class ExtensionDescriptor(object):
         if not extension_attrs_map:
             return
 
-        for resource, attrs in six.iteritems(extension_attrs_map):
+        for resource, attrs in extension_attrs_map.items():
             extended_attrs = extended_attributes.get(resource)
             if extended_attrs:
                 attrs.update(extended_attrs)
@@ -196,7 +197,7 @@ class ActionExtensionController(wsgi.Controller):
     def action(self, request, id):
         input_dict = self._deserialize(request.body,
                                        request.get_content_type())
-        for action_name, handler in six.iteritems(self.action_handlers):
+        for action_name, handler in (self.action_handlers).items():
             if action_name in input_dict:
                 return handler(input_dict, request, id)
         # no action handler found (bump to downstream application)
@@ -238,7 +239,7 @@ class ExtensionController(wsgi.Controller):
 
     def index(self, request):
         extensions = []
-        for _alias, ext in six.iteritems(self.extension_manager.extensions):
+        for _alias, ext in (self.extension_manager.extensions).items():
             extensions.append(self._translate(ext))
         return dict(extensions=extensions)
 
@@ -277,9 +278,8 @@ class ExtensionMiddleware(wsgi.Middleware):
                                (resource.parent["collection_name"],
                                 resource.parent["member_name"]))
 
-            LOG.debug(_('Extended resource: %s'),
-                      resource.collection)
-            for action, method in six.iteritems(resource.collection_actions):
+            LOG.debug('Extended resource: %s', resource.collection)
+            for action, method in (resource.collection_actions).items():
                 conditions = dict(method=[method])
                 path = "/%s/%s" % (resource.collection, action)
                 with mapper.submapper(controller=resource.controller,
@@ -299,7 +299,7 @@ class ExtensionMiddleware(wsgi.Middleware):
         action_controllers = self._action_ext_controllers(application,
                                                           self.ext_mgr, mapper)
         for action in self.ext_mgr.get_actions():
-            LOG.debug(_('Extended action: %s'), action.action_name)
+            LOG.debug('Extended action: %s', action.action_name)
             controller = action_controllers[action.collection]
             controller.add_action(action.action_name, action.handler)
 
@@ -307,7 +307,7 @@ class ExtensionMiddleware(wsgi.Middleware):
         req_controllers = self._request_ext_controllers(application,
                                                         self.ext_mgr, mapper)
         for request_ext in self.ext_mgr.get_request_extensions():
-            LOG.debug(_('Extended request: %s'), request_ext.key)
+            LOG.debug('Extended request: %s', request_ext.key)
             controller = req_controllers[request_ext.key]
             controller.add_handler(request_ext.handler)
 
@@ -405,7 +405,7 @@ class ExtensionManager(object):
         return cls._instance
 
     def __init__(self, path):
-        LOG.info(_('Initializing extension manager.'))
+        LOG.info('Initializing extension manager.')
         self.path = path
         self.extensions = {}
         self._load_all_extensions()
@@ -452,8 +452,8 @@ class ExtensionManager(object):
     def extend_resources(self, version, attr_map):
         """Extend resources with additional resources or attributes.
 
-        :param: attr_map, the existing mapping from resource name to
-        attrs definition.
+        :param attr_map: the existing mapping from resource name to
+                         attrs definition.
 
         After this function, we will extend the attr_map if an extension
         wants to extend this map.
@@ -465,7 +465,7 @@ class ExtensionManager(object):
         # is made in a whole iteration
         while exts_to_process:
             processed_ext_count = len(processed_exts)
-            for ext_name, ext in exts_to_process.items():
+            for ext_name, ext in list(exts_to_process.items()):
                 if not hasattr(ext, 'get_extended_resources'):
                     del exts_to_process[ext_name]
                     continue
@@ -479,15 +479,14 @@ class ExtensionManager(object):
                         continue
                 try:
                     extended_attrs = ext.get_extended_resources(version)
-                    for resource, resource_attrs in six.iteritems(
-                            extended_attrs):
+                    for resource, resource_attrs in extended_attrs.items():
                         if attr_map.get(resource):
                             attr_map[resource].update(resource_attrs)
                         else:
                             attr_map[resource] = resource_attrs
                 except AttributeError:
-                    LOG.exception(_("Error fetching extended attributes for "
-                                    "extension '%s'"), ext.get_name())
+                    LOG.exception("Error fetching extended attributes for "
+                                  "extension '%s'", ext.get_name())
                 processed_exts.add(ext_name)
                 del exts_to_process[ext_name]
             if len(processed_exts) == processed_ext_count:
@@ -495,8 +494,8 @@ class ExtensionManager(object):
                 break
         if exts_to_process:
             # NOTE(salv-orlando): Consider whether this error should be fatal
-            LOG.error(_("It was impossible to process the following "
-                        "extensions: %s because of missing requirements."),
+            LOG.error("It was impossible to process the following "
+                      "extensions: %s because of missing requirements.",
                       ','.join(exts_to_process.keys()))
 
         # Extending extensions' attributes map.
@@ -506,14 +505,13 @@ class ExtensionManager(object):
     def _check_extension(self, extension):
         """Checks for required methods in extension objects."""
         try:
-            LOG.debug(_('Ext name: %s'), extension.get_name())
-            LOG.debug(_('Ext alias: %s'), extension.get_alias())
-            LOG.debug(_('Ext description: %s'), extension.get_description())
-            LOG.debug(_('Ext namespace: %s'), extension.get_namespace())
-            LOG.debug(_('Ext updated: %s'), extension.get_updated())
+            LOG.debug('Ext name: %s', extension.get_name())
+            LOG.debug('Ext alias: %s', extension.get_alias())
+            LOG.debug('Ext description: %s', extension.get_description())
+            LOG.debug('Ext namespace: %s', extension.get_namespace())
+            LOG.debug('Ext updated: %s', extension.get_updated())
         except AttributeError as ex:
-            LOG.exception(_("Exception loading extension: %s"),
-                six.text_type(ex))
+            LOG.exception("Exception loading extension: %s", ex)
             return False
         return True
 
@@ -531,7 +529,7 @@ class ExtensionManager(object):
             if os.path.exists(path):
                 self._load_all_extensions_from_path(path)
             else:
-                LOG.error(_("Extension path '%s' doesn't exist!"), path)
+                LOG.error("Extension path '%s' doesn't exist!", path)
 
     def _load_all_extensions_from_path(self, path):
         # Sorting the extension list makes the order in which they
@@ -539,24 +537,25 @@ class ExtensionManager(object):
         # Tacker Servers
         for f in sorted(os.listdir(path)):
             try:
-                LOG.debug(_('Loading extension file: %s'), f)
+                LOG.debug('Loading extension file: %s', f)
                 mod_name, file_ext = os.path.splitext(os.path.split(f)[-1])
                 ext_path = os.path.join(path, f)
                 if file_ext.lower() == '.py' and not mod_name.startswith('_'):
-                    mod = imp.load_source(mod_name, ext_path)
+                    mod = importutils.import_module(
+                        'tacker.extensions.' + mod_name)
                     ext_name = mod_name[0].upper() + mod_name[1:]
                     new_ext_class = getattr(mod, ext_name, None)
                     if not new_ext_class:
-                        LOG.warning(_('Did not find expected name '
-                                      '"%(ext_name)s" in %(file)s'),
+                        LOG.warning('Did not find expected name '
+                                    '"%(ext_name)s" in %(file)s',
                                     {'ext_name': ext_name,
                                      'file': ext_path})
                         continue
                     new_ext = new_ext_class()
                     self.add_extension(new_ext)
             except Exception as exception:
-                LOG.warning(_("Extension file %(f)s wasn't loaded due to "
-                              "%(exception)s"),
+                LOG.warning("Extension file %(f)s wasn't loaded due to "
+                            "%(exception)s",
                             {'f': f, 'exception': exception})
 
     def add_extension(self, ext):
@@ -565,7 +564,7 @@ class ExtensionManager(object):
             return
 
         alias = ext.get_alias()
-        LOG.info(_('Loaded extension: %s'), alias)
+        LOG.info('Loaded extension: %s', alias)
 
         if alias in self.extensions:
             raise exceptions.DuplicatedExtension(alias=alias)
