@@ -20,14 +20,13 @@ class OpenstackClients(object):
         super(OpenstackClients, self).__init__()
         self.keystone_plugin = keystone.Keystone()
         self.heat_client = None
+        self.mistral_client = None
         self.keystone_client = None
         self.region_name = region_name
         self.auth_attr = auth_attr
 
     def _keystone_client(self):
-        version = self.auth_attr['auth_url'].rpartition('/')[2]
-        return self.keystone_plugin.initialize_client(version,
-                                                      **self.auth_attr)
+        return self.keystone_plugin.initialize_client(**self.auth_attr)
 
     def _heat_client(self):
         endpoint = self.keystone_session.get_endpoint(
