@@ -25,6 +25,7 @@ from oslo_log import log as logging
 import oslo_messaging
 from paste import deploy
 
+from tacker._i18n import _
 from tacker.common import utils
 from tacker import version
 
@@ -32,8 +33,8 @@ from tacker import version
 LOG = logging.getLogger(__name__)
 
 core_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_("The host IP to bind to")),
+    cfg.HostAddressOpt('bind_host', default='0.0.0.0',
+                       help=_("The host IP to bind to")),
     cfg.IntOpt('bind_port', default=9890,
                help=_("The port to bind to")),
     cfg.StrOpt('api_paste_config', default="api-paste.ini",
@@ -42,8 +43,6 @@ core_opts = [
                help=_("The path for API extensions")),
     cfg.ListOpt('service_plugins', default=['nfvo', 'vnfm'],
                 help=_("The service plugins Tacker will use")),
-    cfg.StrOpt('policy_file', default="policy.json",
-               help=_("The policy file to use")),
     cfg.StrOpt('auth_strategy', default='keystone',
                help=_("The type of authentication to use")),
     cfg.BoolOpt('allow_bulk', default=True,
@@ -53,18 +52,19 @@ core_opts = [
     cfg.BoolOpt('allow_sorting', default=False,
                 help=_("Allow the usage of the sorting")),
     cfg.StrOpt('pagination_max_limit', default="-1",
-               help=_("The maximum number of items returned in a single "
-                      "response, value was 'infinite' or negative integer "
-                      "means no limit")),
-    cfg.StrOpt('host', default=utils.get_hostname(),
-               help=_("The hostname Tacker is running on")),
+               help=_("The maximum number of items returned "
+                      "in a single response, value was 'infinite' "
+                      "or negative integer means no limit")),
+    cfg.HostAddressOpt('host', default=utils.get_hostname(),
+                       help=_("The hostname Tacker is running on")),
 ]
 
 core_cli_opts = [
     cfg.StrOpt('state_path',
                default='/var/lib/tacker',
                help=_("Where to store Tacker state files. "
-                      "This directory must be writable by the agent.")),
+                      "This directory must be writable by "
+                      "the agent.")),
 ]
 
 logging.register_options(cfg.CONF)
@@ -110,15 +110,15 @@ def setup_logging(conf):
     """
     product_name = "tacker"
     logging.setup(conf, product_name)
-    LOG.info(_("Logging enabled!"))
+    LOG.info("Logging enabled!")
 
 
 def load_paste_app(app_name):
     """Builds and returns a WSGI app from a paste config file.
 
     :param app_name: Name of the application to load
-    :raises ConfigFilesNotFoundError when config file cannot be located
-    :raises RuntimeError when application cannot be loaded from config file
+    :raises ConfigFilesNotFoundError: when config file cannot be located
+    :raises RuntimeError: when application cannot be loaded from config file
     """
 
     config_path = cfg.CONF.find_file(cfg.CONF.api_paste_config)
@@ -126,7 +126,7 @@ def load_paste_app(app_name):
         raise cfg.ConfigFilesNotFoundError(
             config_files=[cfg.CONF.api_paste_config])
     config_path = os.path.abspath(config_path)
-    LOG.info(_("Config paste file: %s"), config_path)
+    LOG.info("Config paste file: %s", config_path)
 
     try:
         app = deploy.loadapp("config:%s" % config_path, name=app_name)
