@@ -18,6 +18,7 @@ import abc
 from oslo_log import log as logging
 import six
 
+from tacker._i18n import _
 from tacker.api import extensions
 from tacker.api.v1 import attributes as attr
 from tacker.api.v1 import base
@@ -47,20 +48,28 @@ class VNFInUse(exceptions.InUse):
     message = _('VNF %(vnf_id)s is still in use')
 
 
+class InvalidMgmtDriver(exceptions.InvalidInput):
+    message = _('Invalid Mgmt driver %(mgmt_driver_name)s.')
+
+
 class InvalidInfraDriver(exceptions.InvalidInput):
-    message = _('VIM type %(vim_name)s is not supported as an infra driver ')
+    message = _('VIM type %(vim_name)s is not supported as an infra driver')
 
 
-class InvalidServiceType(exceptions.InvalidInput):
-    message = _('invalid service type %(service_type)s')
+class InvalidAPIAttributeType(exceptions.InvalidInput):
+    message = _('Expecting dict type for API attribute instead of %(atype)s ')
 
 
 class VNFCreateFailed(exceptions.TackerException):
     message = _('creating VNF based on %(vnfd_id)s failed')
 
 
+class VNFUpdateWaitFailed(exceptions.TackerException):
+    message = _('VNF Update %(reason)s')
+
+
 class VNFCreateWaitFailed(exceptions.TackerException):
-    message = _('%(reason)s')
+    message = _('VNF Create %(reason)s')
 
 
 class VNFScaleWaitFailed(exceptions.TackerException):
@@ -68,9 +77,21 @@ class VNFScaleWaitFailed(exceptions.TackerException):
 
 
 class VNFDeleteWaitFailed(exceptions.TackerException):
+    message = _('VNF Delete %(reason)s')
+
+
+class VNFHealWaitFailed(exceptions.TackerException):
+    message = _('VNF Heal %(reason)s')
+
+
+class VNFDeleteFailed(exceptions.TackerException):
     message = _('%(reason)s')
 
 
+class VNFHealFailed(exceptions.TackerException):
+    message = _('VNF %(vnf_id)s failed to heal')
+
+
 class VNFDNotFound(exceptions.NotFound):
     message = _('VNFD %(vnfd_id)s could not be found')
 
@@ -99,18 +120,14 @@ class HeatClientException(exceptions.TackerException):
     message = _("%(msg)s")
 
 
-class UserDataFormatNotFound(exceptions.NotFound):
-    message = _("user_data and/or user_data_format not provided")
-
-
 class IPAddrInvalidInput(exceptions.InvalidInput):
     message = _("IP Address input values should be in a list format")
 
 
 class HugePageSizeInvalidInput(exceptions.InvalidInput):
-    message = _("Value specified for mem_page_size is invalid:"
+    message = _("Value specified for mem_page_size is invalid: "
                 "%(error_msg_details)s. The valid values are 'small', 'large',"
-                "'any' or an integer value in MB")
+                " 'any' or an integer value in MB")
 
 
 class CpuAllocationInvalidKeys(exceptions.InvalidInput):
@@ -118,6 +135,11 @@ class CpuAllocationInvalidKeys(exceptions.InvalidInput):
                 "Supported keys are: %(valid_keys)s")
 
 
+class CpuAllocationInvalidValues(exceptions.InvalidInput):
+    message = _("Invalid values specified in VNFD - %(error_msg_details)s."
+                "Supported Values are: %(valid_values)s")
+
+
 class NumaNodesInvalidKeys(exceptions.InvalidInput):
     message = _("Invalid keys specified in VNFD - %(error_msg_details)s."
                 "Supported keys are: %(valid_keys)s")
@@ -134,13 +156,18 @@ class InfraDriverUnreachable(exceptions.ServiceUnavailable):
 
 
 class VNFInactive(exceptions.InvalidInput):
-    message = _("VNF %(vnf_id)s is not in Active state %(message)s")
+    message = _("VNF %(vnf_id)s is not in Active state: %(message)s")
 
 
 class MetadataNotMatched(exceptions.InvalidInput):
     message = _("Metadata for alarm policy is not matched")
 
 
+class InvalidResourceType(exceptions.InvalidInput):
+    message = _("Resource type %(resource_type)s for alarm policy "
+                "is not supported")
+
+
 class InvalidSubstitutionMapping(exceptions.InvalidInput):
     message = _("Input for substitution mapping requirements are not"
                 " valid for %(requirement)s. They must be in the form"
@@ -170,11 +197,11 @@ class InvalidKubernetesInputParameter(exceptions.InvalidInput):
 
 def _validate_service_type_list(data, valid_values=None):
     if not isinstance(data, list):
-        msg = _("invalid data format for service list: '%s'") % data
+        msg = _("Invalid data format for service list: '%s'") % data
         LOG.debug(msg)
         return msg
     if not data:
-        msg = _("empty list is not allowed for service list. '%s'") % data
+        msg = _("Empty list is not allowed for service list. '%s'") % data
         LOG.debug(msg)
         return msg
     key_specs = {
@@ -305,7 +332,7 @@ RESOURCE_ATTRIBUTE_MAP = {
             'validate': {'type:string': None},
             'is_visible': True,
         },
-        'mgmt_url': {
+        'mgmt_ip_address': {
             'allow_post': False,
             'allow_put': False,
             'validate': {'type:string': None},
@@ -470,7 +497,7 @@ class Vnfm(extensions.ExtensionDescriptor):
 
     @classmethod
     def get_namespace(cls):
-        return 'http://wiki.openstack.org/Tacker'
+        return 'https://wiki.openstack.org/Tacker'
 
     @classmethod
     def get_updated(cls):
