@@ -20,7 +20,6 @@ import yaml
 
 from keystoneauth1 import exceptions
 from keystoneauth1 import identity
-from keystoneauth1.identity import v2
 from keystoneauth1.identity import v3
 from keystoneauth1 import session
 from neutronclient.common import exceptions as nc_exceptions
@@ -30,12 +29,14 @@ from oslo_log import log as logging
 
 from tacker._i18n import _
 from tacker.common import log
+from tacker import context as t_context
 from tacker.extensions import nfvo
 from tacker.keymgr import API as KEYMGR_API
 from tacker.mistral import mistral_client
 from tacker.nfvo.drivers.vim import abstract_vim_driver
 from tacker.nfvo.drivers.vnffg import abstract_vnffg_driver
 from tacker.nfvo.drivers.workflow import workflow_generator
+from tacker.nfvo.nfvo_plugin import NfvoPlugin
 from tacker.plugins.common import constants
 from tacker.vnfm import keystone
 
@@ -53,11 +54,11 @@ OPTS = [cfg.StrOpt('openstack', default='/etc/tacker/vim/fernet_keys',
 # same params as we used in ping monitor driver
 OPENSTACK_OPTS = [
     cfg.StrOpt('count', default='1',
-               help=_('number of ICMP packets to send')),
+               help=_('Number of ICMP packets to send')),
     cfg.StrOpt('timeout', default='1',
-               help=_('number of seconds to wait for a response')),
+               help=_('Number of seconds to wait for a response')),
     cfg.StrOpt('interval', default='1',
-               help=_('number of seconds to wait between packets'))
+               help=_('Number of seconds to wait between packets'))
 ]
 cfg.CONF.register_opts(OPTS, 'vim_keys')
 cfg.CONF.register_opts(OPENSTACK_OPTS, 'vim_monitor')
@@ -82,6 +83,7 @@ FC_MAP = {'name': 'name',
           'network_dst_port_id': 'logical_destination_port'}
 
 CONNECTION_POINT = 'connection_points'
+SFC_ENCAP = 'sfc_encap'
 
 
 def config_opts():
@@ -119,66 +121,37 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         verify = 'True' == vim_obj['auth_cred'].get('cert_verify', 'True') \
                  or False
         auth_url = vim_obj['auth_url']
-        keystone_version = self._validate_auth_url(auth_url=auth_url,
-                                                   verify=verify)
-        auth_cred = self._get_auth_creds(keystone_version, vim_obj)
-        return self._initialize_keystone(keystone_version, auth_cred)
+        keystone_version = NfvoPlugin.validate_keystone_auth_url(
+            auth_url=auth_url,
+            verify=verify)
+        auth_cred = self._get_auth_creds(vim_obj, keystone_version)
+        return self._initialize_keystone(auth_cred)
 
-    def _get_auth_creds(self, keystone_version, vim_obj):
-        auth_url = vim_obj['auth_url']
+    def _get_auth_creds(self, vim_obj, keystone_version):
         auth_cred = vim_obj['auth_cred']
         vim_project = vim_obj['vim_project']
-
-        if keystone_version not in auth_url:
-            vim_obj['auth_url'] = auth_url + '/' + keystone_version
-        if keystone_version == 'v3':
-            auth_cred['project_id'] = vim_project.get('id')
-            auth_cred['project_name'] = vim_project.get('name')
-            auth_cred['project_domain_name'] = vim_project.get(
-                'project_domain_name')
-        else:
-            auth_cred['tenant_id'] = vim_project.get('id')
-            auth_cred['tenant_name'] = vim_project.get('name')
-            # pop stuff not supported in keystone v2
-            auth_cred.pop('user_domain_name', None)
-            auth_cred.pop('user_id', None)
+        auth_cred['project_id'] = vim_project.get('id')
+        auth_cred['project_name'] = vim_project.get('name')
+        auth_cred['project_domain_name'] = vim_project.get(
+            'project_domain_name')
         auth_cred['auth_url'] = vim_obj['auth_url']
+        if keystone_version not in auth_cred['auth_url']:
+            auth_cred['auth_url'] = auth_cred['auth_url'] + '/' + \
+                keystone_version
         return auth_cred
 
-    def _get_auth_plugin(self, version, **kwargs):
-        if version == 'v2.0':
-            auth_plugin = v2.Password(**kwargs)
-        else:
-            auth_plugin = v3.Password(**kwargs)
+    def _get_auth_plugin(self, **kwargs):
+        auth_plugin = v3.Password(**kwargs)
 
         return auth_plugin
 
-    def _validate_auth_url(self, auth_url, verify):
-        try:
-            keystone_version = self.keystone.get_version(auth_url, verify)
-        except Exception as e:
-            LOG.error('VIM Auth URL invalid')
-            raise nfvo.VimConnectionException(message=str(e))
-        return keystone_version
-
-    def _initialize_keystone(self, version, auth):
-        ks_client = self.keystone.initialize_client(version=version, **auth)
+    def _initialize_keystone(self, auth):
+        ks_client = self.keystone.initialize_client(**auth)
         return ks_client
 
     def _find_regions(self, ks_client):
-        if ks_client.version == 'v2.0':
-            service_list = ks_client.services.list()
-            heat_service_id = None
-            for service in service_list:
-                if service.type == 'orchestration':
-                    heat_service_id = service.id
-            endpoints_list = ks_client.endpoints.list()
-            region_list = [endpoint.region for endpoint in
-                           endpoints_list if endpoint.service_id ==
-                           heat_service_id]
-        else:
-            region_info = ks_client.regions.list()
-            region_list = [region.id for region in region_info]
+        region_info = ks_client.regions.list()
+        region_list = [region.id for region in region_info]
         return region_list
 
     def discover_placement_attr(self, vim_obj, ks_client):
@@ -195,7 +168,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         return vim_obj
 
     @log.log
-    def register_vim(self, context, vim_obj):
+    def register_vim(self, vim_obj):
         """Validate and set VIM placements."""
 
         if 'key_type' in vim_obj['auth_cred']:
@@ -205,19 +178,19 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         ks_client = self.authenticate_vim(vim_obj)
         self.discover_placement_attr(vim_obj, ks_client)
-        self.encode_vim_auth(context, vim_obj['id'], vim_obj['auth_cred'])
+        self.encode_vim_auth(vim_obj['id'], vim_obj['auth_cred'])
         LOG.debug('VIM registration completed for %s', vim_obj)
 
     @log.log
-    def deregister_vim(self, context, vim_obj):
+    def deregister_vim(self, vim_obj):
         """Deregister VIM from NFVO
 
         Delete VIM keys from file system
         """
-        self.delete_vim_auth(context, vim_obj['id'], vim_obj['auth_cred'])
+        self.delete_vim_auth(vim_obj['id'], vim_obj['auth_cred'])
 
     @log.log
-    def delete_vim_auth(self, context, vim_id, auth):
+    def delete_vim_auth(self, vim_id, auth):
         """Delete vim information
 
         Delete vim key stored in file system
@@ -226,10 +199,11 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         if auth.get('key_type') == 'barbican_key':
             try:
+                k_context = t_context.generate_tacker_service_context()
                 keystone_conf = CONF.keystone_authtoken
                 secret_uuid = auth['secret_uuid']
                 keymgr_api = KEYMGR_API(keystone_conf.auth_url)
-                keymgr_api.delete(context, secret_uuid)
+                keymgr_api.delete(k_context, secret_uuid)
                 LOG.debug('VIM key deleted successfully for vim %s',
                           vim_id)
             except Exception as ex:
@@ -248,7 +222,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                             vim_id)
 
     @log.log
-    def encode_vim_auth(self, context, vim_id, auth):
+    def encode_vim_auth(self, vim_id, auth):
         """Encode VIM credentials
 
          Store VIM auth using fernet key encryption
@@ -259,9 +233,10 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         if CONF.vim_keys.use_barbican:
             try:
+                k_context = t_context.generate_tacker_service_context()
                 keystone_conf = CONF.keystone_authtoken
                 keymgr_api = KEYMGR_API(keystone_conf.auth_url)
-                secret_uuid = keymgr_api.store(context, fernet_key)
+                secret_uuid = keymgr_api.store(k_context, fernet_key)
 
                 auth['key_type'] = 'barbican_key'
                 auth['secret_uuid'] = secret_uuid
@@ -336,10 +311,11 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         """
         verify = 'True' == vim_obj.get('cert_verify', 'True') or False
         auth_url = vim_obj['auth_url']
-        keystone_version = self._validate_auth_url(auth_url=auth_url,
-                                                   verify=verify)
-        auth_cred = self._get_auth_creds(keystone_version, vim_obj)
-        auth_plugin = self._get_auth_plugin(keystone_version, **auth_cred)
+        keystone_version = NfvoPlugin.validate_keystone_auth_url(
+            auth_url=auth_url,
+            verify=verify)
+        auth_cred = self._get_auth_creds(vim_obj, keystone_version)
+        auth_plugin = self._get_auth_plugin(**auth_cred)
         sess = session.Session(auth=auth_plugin)
         return client_type(session=sess)
 
@@ -387,69 +363,120 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         raise ValueError('empty match field for input flow classifier')
 
-    def create_chain(self, name, fc_ids, vnfs, symmetrical=False,
-                     auth_attr=None):
+    def create_chain(self, name, path_id, fc_ids, vnfs, symmetrical=False,
+                     correlation='mpls', auth_attr=None):
         if not auth_attr:
             LOG.warning("auth information required for n-sfc driver")
             return None
 
         neutronclient_ = NeutronClient(auth_attr)
+        port_pairs_list = neutronclient_.port_pair_list()
+        port_pair_groups_list = neutronclient_.port_pair_group_list()
+        port_chains_list = neutronclient_.port_chain_list()
         port_pair_group_list = []
-        for vnf in vnfs:
-            # TODO(s3wong): once scaling is in place and VNFFG supports it
-            # that model needs to be implemented to concatenate all
-            # port-pairs into the port-pair-group
-            # port pair group could include port-pairs from different VNFs
-            port_pair_group = {}
-            port_pair_group['name'] = vnf['name'] + '-port-pair-group'
-            port_pair_group['description'] = \
-                'port pair group for %s' % vnf['name']
-            port_pair_group['port_pairs'] = []
-            if CONNECTION_POINT not in vnf:
-                LOG.warning("Chain creation failed due to missing "
-                            "connection point info in VNF "
-                            "%(vnfname)s", {'vnfname': vnf['name']})
-                return None
-            cp_list = vnf[CONNECTION_POINT]
-            num_cps = len(cp_list)
-            if num_cps != 1 and num_cps != 2:
-                LOG.warning("Chain creation failed due to wrong number of "
-                            "connection points: expected [1 | 2], got "
-                            "%(cps)d", {'cps': num_cps})
-                return None
-            port_pair = {}
-            port_pair['name'] = vnf['name'] + '-connection-points'
-            port_pair['description'] = 'port pair for %s' % vnf['name']
-            if num_cps == 1:
-                port_pair['ingress'] = cp_list[0]
-                port_pair['egress'] = cp_list[0]
-            else:
-                port_pair['ingress'] = cp_list[0]
-                port_pair['egress'] = cp_list[1]
-            port_pair_id = neutronclient_.port_pair_create(port_pair)
-            if not port_pair_id:
-                LOG.warning("Chain creation failed due to port pair creation"
-                            " failed for vnf %(vnf)s", {'vnf': vnf['name']})
-                return None
-            port_pair_group['port_pairs'].append(port_pair_id)
-            port_pair_group_id = \
-                neutronclient_.port_pair_group_create(port_pair_group)
-            if not port_pair_group_id:
-                LOG.warning("Chain creation failed due to port pair group "
-                            "creation failed for vnf "
-                            "%(vnf)s", {'vnf': vnf['name']})
-                return None
-            port_pair_group_list.append(port_pair_group_id)
+        new_ppgs = []
+        new_pps = []
+
+        try:
+            for vnf in vnfs:
+                # TODO(s3wong): once scaling is in place and VNFFG supports it
+                # that model needs to be implemented to concatenate all
+                # port-pairs into the port-pair-group
+                # port pair group could include port-pairs from different VNFs
+                if CONNECTION_POINT not in vnf:
+                    LOG.warning("Chain creation failed due to missing "
+                                "connection point info in VNF "
+                                "%(vnfname)s", {'vnfname': vnf['name']})
+                    return None
+                cp_list = vnf[CONNECTION_POINT]
+                num_cps = len(cp_list)
+                if num_cps not in [1, 2]:
+                    LOG.warning("Chain creation failed due to wrong number of "
+                                "connection points: expected [1 | 2], got "
+                                "%(cps)d", {'cps': num_cps})
+                    return None
+                if num_cps == 1:
+                    ingress = cp_list[0]
+                    egress = cp_list[0]
+                else:
+                    ingress = cp_list[0]
+                    egress = cp_list[1]
+
+                # If sfc_encap is True, pp_corr is set to correlation to
+                # make use of correlation, otherwise pp_corr is set to None
+                # to install SFC proxy
+                sfc_encap = vnf.get(SFC_ENCAP, True)
+                pp_corr = correlation if sfc_encap else None
+
+                # valid_port_in_use function is used to find out the
+                # port_pair_group_id of the existing port pair group
+                # which was created by ingress and egress of current VNF
+                port_pair_group_id = self.valid_port_in_use(
+                    ingress, egress, port_pairs_list, port_pair_groups_list)
+                if not port_pair_group_id:
+                    # create the new port pair group if it is not existed
+                    port_pair = dict()
+                    port_pair['name'] = vnf['name'] + '-connection-points'
+                    port_pair['description'] = 'port pair for ' + vnf['name']
+                    port_pair['ingress'] = ingress
+                    port_pair['egress'] = egress
+                    port_pair['service_function_parameters'] = {
+                        'correlation': pp_corr}
+                    port_pair_id = neutronclient_.port_pair_create(port_pair)
+                    if not port_pair_id:
+                        LOG.warning("Chain creation failed due to port pair "
+                                    "creation failed for vnf %(vnf)s",
+                                    {'vnf': vnf['name']})
+                        return None
+                    new_pps.append(port_pair_id)
+                    port_pair_group = {}
+                    port_pair_group['name'] = vnf['name'] + '-port-pair-group'
+                    port_pair_group['description'] = \
+                        'port pair group for ' + vnf['name']
+                    port_pair_group['port_pairs'] = []
+                    port_pair_group['port_pairs'].append(port_pair_id)
+                    port_pair_group_id = \
+                        neutronclient_.port_pair_group_create(port_pair_group)
+                    new_ppgs.append(port_pair_group_id)
+                if not port_pair_group_id:
+                    LOG.warning("Chain creation failed due to port pair group "
+                                "creation failed for vnf "
+                                "%(vnf)s", {'vnf': vnf['name']})
+                    raise nfvo.CreateChainException(
+                        message="Failed to create port-pair-group")
+                port_pair_group_list.append(port_pair_group_id)
+
+            # Check list port pair group between new port chain and the
+            # existing port chains.  Networking-sfc does not allow to create
+            # two port chains with the same port pair groups and the same order
+            for pc in port_chains_list['port_chains']:
+                ppg_list = pc['port_pair_groups']
+                if ppg_list == port_pair_group_list:
+                    # raise exception when the Vnffg path is already existing
+                    raise nfvo.CreateChainException(
+                        message="Vnffg path already exists")
+        except nfvo.CreateChainException as e:
+            # clean neutron resources such as port pair, port pair group and
+            # flow classifier if we create it
+            for ppg in new_ppgs:
+                neutronclient_.port_pair_group_delete(ppg)
+            for pp in new_pps:
+                neutronclient_.port_pair_delete(pp)
+            for fc_id in fc_ids:
+                neutronclient_.flow_classifier_delete(fc_id)
+            raise e
 
         # TODO(s3wong): should the chain name be given as a parameter?
         port_chain = {}
         port_chain['name'] = name + '-port-chain'
+        if path_id:
+            port_chain['chain_id'] = path_id
         port_chain['description'] = 'port-chain for Tacker VNFFG'
         port_chain['port_pair_groups'] = port_pair_group_list
         port_chain['flow_classifiers'] = fc_ids
-        if symmetrical:
-            port_chain['chain_parameters'] = {}
-            port_chain['chain_parameters']['symmetric'] = True
+        port_chain['chain_parameters'] = {}
+        port_chain['chain_parameters']['symmetric'] = symmetrical
+        port_chain['chain_parameters']['correlation'] = correlation
         return neutronclient_.port_chain_create(port_chain)
 
     def update_chain(self, chain_id, fc_ids, vnfs,
@@ -473,8 +500,12 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
             return None
 
         neutronclient_ = NeutronClient(auth_attr)
+        port_pairs_list = neutronclient_.port_pair_list()
+        port_pair_groups_list = neutronclient_.port_pair_group_list()
+        port_chains_list = neutronclient_.port_chain_list()
         new_ppgs = []
         updated_port_chain = dict()
+
         pc_info = neutronclient_.port_chain_show(chain_id)
         if set(fc_ids) != set(pc_info['port_chain']['flow_classifiers']):
             updated_port_chain['flow_classifiers'] = fc_ids
@@ -491,11 +522,6 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                     old_ppg_id = old_ppgs_dict.pop(vnf['name'])
                     new_ppgs.append(old_ppg_id)
                 else:
-                    port_pair_group['name'] = vnf['name'] + '-port-pair-group'
-                    port_pair_group['description'] = \
-                        'port pair group for %s' % vnf['name']
-                    port_pair_group['port_pairs'] = []
-
                     if CONNECTION_POINT not in vnf:
                         LOG.warning("Chain update failed due to missing "
                                     "connection point info in VNF "
@@ -504,30 +530,47 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                             message="Connection point not found")
                     cp_list = vnf[CONNECTION_POINT]
                     num_cps = len(cp_list)
-                    if num_cps != 1 and num_cps != 2:
+                    if num_cps not in [1, 2]:
                         LOG.warning("Chain update failed due to wrong number "
                                     "of connection points: expected [1 | 2],"
                                     "got %(cps)d", {'cps': num_cps})
                         raise nfvo.UpdateChainException(
                             message="Invalid number of connection points")
-                    port_pair['name'] = vnf['name'] + '-connection-points'
-                    port_pair['description'] = 'port pair for %s' % vnf['name']
                     if num_cps == 1:
-                        port_pair['ingress'] = cp_list[0]
-                        port_pair['egress'] = cp_list[0]
+                        ingress = cp_list[0]
+                        egress = cp_list[0]
                     else:
-                        port_pair['ingress'] = cp_list[0]
-                        port_pair['egress'] = cp_list[1]
-                    port_pair_id = neutronclient_.port_pair_create(port_pair)
-                    if not port_pair_id:
-                        LOG.warning("Chain update failed due to port pair "
-                                    "creation failed for "
-                                    "vnf %(vnf)s", {'vnf': vnf['name']})
-                        raise nfvo.UpdateChainException(
-                            message="Failed to create port-pair")
-                    port_pair_group['port_pairs'].append(port_pair_id)
-                    port_pair_group_id = \
-                        neutronclient_.port_pair_group_create(port_pair_group)
+                        ingress = cp_list[0]
+                        egress = cp_list[1]
+
+                    # valid_port_in_use function is used to find out the
+                    # port_pair_group_id of the existing port pair group
+                    # which was created by ingress and egress of current VNF
+                    port_pair_group_id = self.valid_port_in_use(
+                        ingress, egress, port_pairs_list,
+                        port_pair_groups_list)
+                    if not port_pair_group_id:
+                        port_pair['name'] = vnf['name'] + '-connection-points'
+                        port_pair['description'] = \
+                            'port pair for ' + vnf['name']
+                        port_pair['ingress'] = ingress
+                        port_pair['egress'] = egress
+                        port_pair_id = neutronclient_.port_pair_create(
+                            port_pair)
+                        if not port_pair_id:
+                            LOG.warning("Chain update failed due to port pair "
+                                        "creation failed for "
+                                        "vnf %(vnf)s", {'vnf': vnf['name']})
+                            raise nfvo.UpdateChainException(
+                                message="Failed to create port-pair")
+                        port_pair_group['name'] = \
+                            vnf['name'] + '-port-pair-group'
+                        port_pair_group['description'] = \
+                            'port pair group for ' + vnf['name']
+                        port_pair_group['port_pairs'] = []
+                        port_pair_group['port_pairs'].append(port_pair_id)
+                        port_pair_group_id = neutronclient_.\
+                            port_pair_group_create(port_pair_group)
                     if not port_pair_group_id:
                         LOG.warning("Chain update failed due to port pair "
                                     "group creation failed for vnf "
@@ -537,8 +580,15 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                         raise nfvo.UpdateChainException(
                             message="Failed to create port-pair-group")
                     new_ppgs.append(port_pair_group_id)
+            for pc in port_chains_list['port_chains']:
+                ppg_list = pc['port_pair_groups']
+                if ppg_list == new_ppgs:
+                    # raise exception when the Vnffg path already exists
+                    nfvo.UpdateChainException(
+                        message="Vnffg path already exists")
         except nfvo.UpdateChainException as e:
-            self._delete_ppgs_and_pps(neutronclient_, new_ppgs, past_ppgs_dict)
+            self._delete_ppgs_and_pps(neutronclient_, new_ppgs,
+                                      past_ppgs_dict, port_chains_list, fc_ids)
             raise e
 
         updated_port_chain['port_pair_groups'] = new_ppgs
@@ -547,17 +597,21 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
             pc_id = neutronclient_.port_chain_update(chain_id,
                                                      updated_port_chain)
         except (nc_exceptions.BadRequest, nfvo.UpdateChainException) as e:
-            self._delete_ppgs_and_pps(neutronclient_, new_ppgs, past_ppgs_dict)
+            self._delete_ppgs_and_pps(neutronclient_, new_ppgs,
+                                      past_ppgs_dict, port_chains_list)
             raise e
         for ppg_name in old_ppgs_dict:
             ppg_info = neutronclient_. \
                 port_pair_group_show(old_ppgs_dict[ppg_name])
-            neutronclient_.port_pair_group_delete(old_ppgs_dict[ppg_name])
-            port_pairs = ppg_info['port_pair_group']['port_pairs']
-            if port_pairs and len(port_pairs):
-                for j in xrange(0, len(port_pairs)):
-                    pp_id = port_pairs[j]
-                    neutronclient_.port_pair_delete(pp_id)
+            ppg_inuse = self.valid_ppg_for_multiple_chain(
+                ppg_info['port_pair_group']['id'], port_chains_list)
+            if not ppg_inuse:
+                neutronclient_.port_pair_group_delete(old_ppgs_dict[ppg_name])
+                port_pairs = ppg_info['port_pair_group']['port_pairs']
+                if port_pairs and len(port_pairs):
+                    for j in range(0, len(port_pairs)):
+                        pp_id = port_pairs[j]
+                        neutronclient_.port_pair_delete(pp_id)
         return pc_id
 
     def delete_chain(self, chain_id, auth_attr=None):
@@ -667,7 +721,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
             raise EnvironmentError('auth dict required for'
                                    ' mistral workflow driver')
         return mistral_client.MistralClient(
-            keystone.Keystone().initialize_client('2', **auth_dict),
+            keystone.Keystone().initialize_client(**auth_dict),
             auth_dict['token']).get_client()
 
     def prepare_and_create_workflow(self, resource, action,
@@ -694,25 +748,60 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
     def delete_execution(self, execution_id, auth_dict=None):
         return self.get_mistral_client(auth_dict).executions \
-            .delete(execution_id)
+            .delete(execution_id, force=True)
 
     def delete_workflow(self, workflow_id, auth_dict=None):
         return self.get_mistral_client(auth_dict) \
             .workflows.delete(workflow_id)
 
-    def _delete_ppgs_and_pps(self, neutronclient, new_ppgs, past_ppgs_dict):
+    def _delete_ppgs_and_pps(self, neutronclient, new_ppgs,
+                             past_ppgs_dict, pcs_list, fc_ids):
         if new_ppgs:
             for item in new_ppgs:
                 if item not in past_ppgs_dict.values():
-                    new_ppg_info = neutronclient.port_pair_group_show(
-                        item)
-                    neutronclient.port_pair_group_delete(item)
-                    new_port_pairs = new_ppg_info['port_pair_group'][
-                        'port_pairs']
-                    if new_port_pairs and len(new_port_pairs):
-                        for j in xrange(0, len(new_port_pairs)):
-                            new_pp_id = new_port_pairs[j]
-                            neutronclient.port_pair_delete(new_pp_id)
+                    ppg_inuse = self.valid_ppg_for_multiple_chain(
+                        item, pcs_list)
+                    if not ppg_inuse:
+                        # clean port pair and port pair group if
+                        # it is not in used
+                        new_ppg_info = neutronclient.port_pair_group_show(item)
+                        neutronclient.port_pair_group_delete(item)
+                        new_port_pairs = new_ppg_info['port_pair_group'][
+                            'port_pairs']
+                        if new_port_pairs and len(new_port_pairs):
+                            for j in range(0, len(new_port_pairs)):
+                                new_pp_id = new_port_pairs[j]
+                                neutronclient.port_pair_delete(new_pp_id)
+        # clean flow classifiers
+        for fc_id in fc_ids:
+            neutronclient.flow_classifier_delete(fc_id)
+
+    def valid_port_in_use(self, ingress, egress, pps_list, ppgs_list):
+        # This function checks the the ports are used or not and return the
+        # port pair group id of these ports
+        port_pair_list = pps_list['port_pairs']
+        port_pair_group_list = ppgs_list['port_pair_groups']
+        port_pair_id = None
+        port_pair_group_id = None
+        for pp in port_pair_list:
+            if (ingress == pp['ingress']) and (egress == pp['egress']):
+                port_pair_id = pp['id']
+                break
+        if port_pair_id:
+            for ppg in port_pair_group_list:
+                if port_pair_id in ppg['port_pairs']:
+                    port_pair_group_id = ppg['id']
+                    break
+        return port_pair_group_id
+
+    def valid_ppg_for_multiple_chain(self, ppg_id, pcs_list):
+        # This function returns True if a ppg belongs to more than one
+        # port chain. If not return False.
+        count = 0
+        for pc in pcs_list['port_chains']:
+            if ppg_id in pc['port_pair_groups']:
+                count = count + 1
+        return True if count > 1 else False
 
 
 class NeutronClient(object):
@@ -727,7 +816,7 @@ class NeutronClient(object):
 
     def flow_classifier_show(self, fc_id):
         try:
-            fc = self.client.show_flow_classifier(fc_id)
+            fc = self.client.show_sfc_flow_classifier(fc_id)
             if fc is None:
                 raise ValueError('classifier %s not found' % fc_id)
             return fc
@@ -737,26 +826,29 @@ class NeutronClient(object):
 
     def flow_classifier_create(self, fc_dict):
         LOG.debug("fc_dict passed is {fc_dict}".format(fc_dict=fc_dict))
-        fc = self.client.create_flow_classifier({'flow_classifier': fc_dict})
-        if fc:
+        try:
+            fc = self.client.create_sfc_flow_classifier(
+                {'flow_classifier': fc_dict})
             return fc['flow_classifier']['id']
-        else:
-            return None
+        except Exception as ex:
+            LOG.error("Error while creating Flow Classifier: %s", str(ex))
+            raise nfvo.FlowClassiferCreationFailed(message=str(ex))
 
     def flow_classifier_update(self, fc_id, update_fc):
         update_fc_dict = {'flow_classifier': update_fc}
-        return self.client.update_flow_classifier(fc_id, update_fc_dict)
+        return self.client.update_sfc_flow_classifier(fc_id, update_fc_dict)
 
     def flow_classifier_delete(self, fc_id):
         try:
-            self.client.delete_flow_classifier(fc_id)
+            self.client.delete_sfc_flow_classifier(fc_id)
         except nc_exceptions.NotFound:
             LOG.warning("fc %s not found", fc_id)
             raise ValueError('fc %s not found' % fc_id)
 
     def port_pair_create(self, port_pair_dict):
         try:
-            pp = self.client.create_port_pair({'port_pair': port_pair_dict})
+            pp = self.client.create_sfc_port_pair(
+                {'port_pair': port_pair_dict})
         except nc_exceptions.BadRequest as e:
             LOG.error("create port pair returns %s", e)
             raise ValueError(str(e))
@@ -766,16 +858,20 @@ class NeutronClient(object):
         else:
             return None
 
+    def port_pair_list(self):
+        pp_list = self.client.list_sfc_port_pairs()
+        return pp_list
+
     def port_pair_delete(self, port_pair_id):
         try:
-            self.client.delete_port_pair(port_pair_id)
+            self.client.delete_sfc_port_pair(port_pair_id)
         except nc_exceptions.NotFound:
             LOG.warning('port pair %s not found', port_pair_id)
             raise ValueError('port pair %s not found' % port_pair_id)
 
     def port_pair_group_create(self, ppg_dict):
         try:
-            ppg = self.client.create_port_pair_group(
+            ppg = self.client.create_sfc_port_pair_group(
                 {'port_pair_group': ppg_dict})
         except nc_exceptions.BadRequest as e:
             LOG.warning('create port pair group returns %s', e)
@@ -786,49 +882,67 @@ class NeutronClient(object):
         else:
             return None
 
+    def port_pair_group_list(self):
+        ppg_list = self.client.list_sfc_port_pair_groups()
+        return ppg_list
+
     def port_pair_group_delete(self, ppg_id):
         try:
-            self.client.delete_port_pair_group(ppg_id)
+            self.client.delete_sfc_port_pair_group(ppg_id)
         except nc_exceptions.NotFound:
             LOG.warning('port pair group %s not found', ppg_id)
             raise ValueError('port pair group %s not found' % ppg_id)
 
     def port_chain_create(self, port_chain_dict):
         try:
-            pc = self.client.create_port_chain(
+            pc = self.client.create_sfc_port_chain(
                 {'port_chain': port_chain_dict})
         except nc_exceptions.BadRequest as e:
             LOG.warning('create port chain returns %s', e)
             raise ValueError(str(e))
 
         if pc and len(pc):
-            return pc['port_chain']['id']
+            return pc['port_chain']['id'], pc['port_chain']['chain_id']
         else:
             return None
 
     def port_chain_delete(self, port_chain_id):
         try:
-            port_chain = self.client.show_port_chain(port_chain_id)
+            port_chain = self.client.show_sfc_port_chain(port_chain_id)
             if port_chain:
-                self.client.delete_port_chain(port_chain_id)
+                self.client.delete_sfc_port_chain(port_chain_id)
+                port_chain_list = \
+                    self.client.list_sfc_port_chains()['port_chains']
                 ppg_list = port_chain['port_chain'].get('port_pair_groups')
                 if ppg_list and len(ppg_list):
-                    for i in xrange(0, len(ppg_list)):
-                        ppg = self.client.show_port_pair_group(ppg_list[i])
-                        if ppg:
-                            self.client.delete_port_pair_group(ppg_list[i])
-                            port_pairs = ppg['port_pair_group']['port_pairs']
-                            if port_pairs and len(port_pairs):
-                                for j in xrange(0, len(port_pairs)):
-                                    pp_id = port_pairs[j]
-                                    self.client.delete_port_pair(pp_id)
+                    for i in range(0, len(ppg_list)):
+                        ppg_in_use = False
+                        # Firstly, Tacker delete port chain, if a port pair
+                        # group still belong to other port chains, Tacker
+                        # will mark it as in_use and does not delete it.
+                        for pc in port_chain_list:
+                            if ppg_list[i] in pc['port_pair_groups']:
+                                ppg_in_use = True
+                                break
+                        if not ppg_in_use:
+                            ppg = self.client.show_sfc_port_pair_group(
+                                ppg_list[i])
+                            if ppg:
+                                self.client.delete_sfc_port_pair_group(
+                                    ppg_list[i])
+                                port_pairs = \
+                                    ppg['port_pair_group']['port_pairs']
+                                if port_pairs and len(port_pairs):
+                                    for j in range(0, len(port_pairs)):
+                                        pp_id = port_pairs[j]
+                                        self.client.delete_sfc_port_pair(pp_id)
         except nc_exceptions.NotFound:
             LOG.warning('port chain %s not found', port_chain_id)
             raise ValueError('port chain %s not found' % port_chain_id)
 
     def port_chain_update(self, port_chain_id, port_chain):
         try:
-            pc = self.client.update_port_chain(port_chain_id,
+            pc = self.client.update_sfc_port_chain(port_chain_id,
                                     {'port_chain': port_chain})
         except nc_exceptions.BadRequest as e:
             LOG.warning('update port chain returns %s', e)
@@ -839,9 +953,13 @@ class NeutronClient(object):
             raise nfvo.UpdateChainException(message="Failed to update "
                                                     "port-chain")
 
+    def port_chain_list(self):
+        pc_list = self.client.list_sfc_port_chains()
+        return pc_list
+
     def port_chain_show(self, port_chain_id):
         try:
-            port_chain = self.client.show_port_chain(port_chain_id)
+            port_chain = self.client.show_sfc_port_chain(port_chain_id)
             if port_chain is None:
                 raise ValueError('port chain %s not found' % port_chain_id)
 
@@ -852,7 +970,7 @@ class NeutronClient(object):
 
     def port_pair_group_show(self, ppg_id):
         try:
-            port_pair_group = self.client.show_port_pair_group(ppg_id)
+            port_pair_group = self.client.show_sfc_port_pair_group(ppg_id)
             if port_pair_group is None:
                 raise ValueError('port pair group %s not found' % ppg_id)
 
