@@ -12,6 +12,7 @@
 
 POLICY_ALARMING = 'tosca.policies.tacker.Alarming'
 DEFAULT_ALARM_ACTIONS = ['respawn', 'log', 'log_and_kill', 'notify']
+POLICY_RESERVATION = 'tosca.policies.tacker.Reservation'
 VNF_CIRROS_CREATE_TIMEOUT = 300
 VNFC_CREATE_TIMEOUT = 600
 VNF_CIRROS_DELETE_TIMEOUT = 300
@@ -19,6 +20,21 @@ VNF_CIRROS_DEAD_TIMEOUT = 500
 ACTIVE_SLEEP_TIME = 3
 DEAD_SLEEP_TIME = 1
 SCALE_WINDOW_SLEEP_TIME = 120
+SCALE_SLEEP_TIME = 120
 NS_CREATE_TIMEOUT = 400
 NS_DELETE_TIMEOUT = 300
 NOVA_CLIENT_VERSION = 2
+VDU_MARK_UNHEALTHY_TIMEOUT = 500
+VDU_MARK_UNHEALTHY_SLEEP_TIME = 3
+VDU_AUTOHEALING_TIMEOUT = 500
+VDU_AUTOHEALING_SLEEP_TIME = 3
+VNF_CIRROS_PENDING_HEAL_TIMEOUT = 300
+PENDING_SLEEP_TIME = 3
+
+# Blazar related
+LEASE_EVENT_STATUS = 'DONE'
+START_LEASE_EVET_TYPE = 'start_lease'
+LEASE_CHECK_EVENT_TIMEOUT = 300
+LEASE_CHECK_SLEEP_TIME = 3
+UUID = 'f26f181d-7891-4720-b022-b074ec1733ef'
+INVALID_UUID = 'f181d-7891-4720-b022-b074ec3ef'
