@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
+import unittest
 import yaml
 
 from novaclient import exceptions
@@ -30,12 +32,10 @@ VNF_CIRROS_CREATE_TIMEOUT = 120
 class VnfTestToscaCreate(base.BaseTackerTest):
     def _test_create_vnf(self, vnfd_file, vnf_name,
                          template_source="onboarded"):
-        data = dict()
-        values_str = read_file(vnfd_file)
-        data['tosca'] = values_str
-        toscal = data['tosca']
+        input_yaml = read_file(vnfd_file)
+        tosca_dict = yaml.safe_load(input_yaml)
         tosca_arg = {'vnfd': {'name': vnf_name,
-                              'attributes': {'vnfd': toscal}}}
+                              'attributes': {'vnfd': tosca_dict}}}
 
         if template_source == "onboarded":
             # Create vnfd with tosca template
@@ -50,8 +50,7 @@ class VnfTestToscaCreate(base.BaseTackerTest):
 
         if template_source == 'inline':
             # create vnf directly from template
-            template = yaml.load(values_str)
-            vnf_arg = {'vnf': {'vnfd_template': template, 'name': vnf_name}}
+            vnf_arg = {'vnf': {'vnfd_template': tosca_dict, 'name': vnf_name}}
             vnf_instance = self.client.create_vnf(body=vnf_arg)
             vnfd_id = vnf_instance['vnf']['vnfd_id']
 
@@ -61,22 +60,21 @@ class VnfTestToscaCreate(base.BaseTackerTest):
             constants.VNF_CIRROS_CREATE_TIMEOUT,
             constants.ACTIVE_SLEEP_TIME)
         vnf_show_out = self.client.show_vnf(vnf_id)['vnf']
-        self.assertIsNotNone(vnf_show_out['mgmt_url'])
+        self.assertIsNotNone(vnf_show_out['mgmt_ip_address'])
 
-        input_dict = yaml.load(values_str)
-        prop_dict = input_dict['topology_template']['node_templates'][
+        prop_dict = tosca_dict['topology_template']['node_templates'][
             'CP1']['properties']
 
         # Verify if ip_address is static, it is same as in show_vnf
         if prop_dict.get('ip_address'):
-            mgmt_url_input = prop_dict.get('ip_address')
-            mgmt_info = yaml.load(
-                vnf_show_out['mgmt_url'])
-            self.assertEqual(mgmt_url_input, mgmt_info['VDU1'])
+            mgmt_ip_address_input = prop_dict.get('ip_address')
+            mgmt_info = yaml.safe_load(
+                vnf_show_out['mgmt_ip_address'])
+            self.assertEqual(mgmt_ip_address_input, mgmt_info['VDU1'])
 
         # Verify anti spoofing settings
         stack_id = vnf_show_out['instance_id']
-        template_dict = input_dict['topology_template']['node_templates']
+        template_dict = tosca_dict['topology_template']['node_templates']
         for field in template_dict.keys():
             prop_dict = template_dict[field]['properties']
             if prop_dict.get('anti_spoofing_protection'):
@@ -137,6 +135,15 @@ class VnfTestToscaCreate(base.BaseTackerTest):
                                            'test_tosca_vnf_with_cirros_inline',
                                            'inline')
 
+    def test_re_create_delete_vnf(self):
+        self._test_create_delete_vnf_tosca('sample-tosca-vnfd.yaml',
+                                           'test_vnf',
+                                           'inline')
+        time.sleep(1)
+        self._test_create_delete_vnf_tosca('sample-tosca-vnfd.yaml',
+                                           'test_vnf',
+                                           'inline')
+
     def test_create_delete_vnf_static_ip(self):
         vnfd_id, vnf_id = self._test_create_vnf(
             'sample-tosca-vnfd-static-ip.yaml',
@@ -170,7 +177,8 @@ class VnfTestToscaCreateFlavorCreation(base.BaseTackerTest):
             vnf_id,
             constants.VNF_CIRROS_CREATE_TIMEOUT,
             constants.ACTIVE_SLEEP_TIME)
-        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf']['mgmt_url'])
+        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf'][
+            'mgmt_ip_address'])
 
         self.verify_vnf_crud_events(
             vnf_id, evt_constants.RES_EVT_CREATE,
@@ -208,6 +216,8 @@ class VnfTestToscaCreateFlavorCreation(base.BaseTackerTest):
 
 
 class VnfTestToscaCreateImageCreation(base.BaseTackerTest):
+
+    @unittest.skip("Until BUG 1673099")
     def test_create_delete_vnf_tosca_no_monitoring(self):
         vnfd_name = 'tosca_vnfd_with_auto_image'
         input_yaml = read_file('sample-tosca-vnfd-image.yaml')
@@ -232,7 +242,8 @@ class VnfTestToscaCreateImageCreation(base.BaseTackerTest):
             vnf_id,
             constants.VNF_CIRROS_CREATE_TIMEOUT,
             constants.ACTIVE_SLEEP_TIME)
-        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf']['mgmt_url'])
+        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf'][
+            'mgmt_ip_address'])
 
         self.verify_vnf_crud_events(
             vnf_id, evt_constants.RES_EVT_CREATE,
